@@ -4,9 +4,7 @@ import numpy as np
 from tqdm import trange, tqdm
 from urllib.parse import urlparse
 import tempfile
-from scipy.ndimage import median_filter
 import cv2
-
 from . import transforms, dynamics, utils, plot, metrics
 
 try:
@@ -21,10 +19,12 @@ except:
 
 try:
     import torch
-    from torch import optim, nn
+#     from GPUtil import showUtilization as gpu_usage #for gpu memory debugging 
+    from torch import nn
+    import torch_optimizer as optim # for RADAM optimizer
     from torch.utils import mkldnn as mkldnn_utils
     from . import resnet_torch
-    TORCH_ENABLED = True 
+    TORCH_ENABLED = True
     torch_GPU = torch.device('cuda')
     torch_CPU = torch.device('cpu')
 except:
@@ -34,6 +34,7 @@ core_logger = logging.getLogger(__name__)
 core_logger.setLevel(logging.DEBUG)
 tqdm_out = utils.TqdmToLogger(core_logger, level=logging.INFO)
 
+# nclasses now specified by user or by model type in models.py
 def parse_model_string(pretrained_model):
     if isinstance(pretrained_model, list):
         model_str = os.path.split(pretrained_model[0])[-1]
@@ -44,14 +45,13 @@ def parse_model_string(pretrained_model):
         nclasses = max(2, int(model_str[4]))
     elif len(model_str)>7 and model_str[:8]=='cellpose':
         core_logger.info(f'parsing model string {model_str} to get cellpose options')
-        nclasses = 3
     else:
         return None
     ostrs = model_str.split('_')[2::2]
     residual_on = ostrs[0]=='on'
     style_on = ostrs[1]=='on'
     concatenation = ostrs[2]=='on'
-    return nclasses, residual_on, style_on, concatenation
+    return residual_on, style_on, concatenation
 
 def use_gpu(gpu_number=0, istorch=True):
     """ check if gpu works """
@@ -118,7 +118,7 @@ class UnetModel():
     def __init__(self, gpu=False, pretrained_model=False,
                     diam_mean=30., net_avg=True, device=None,
                     residual_on=False, style_on=False, concatenation=True,
-                    nclasses = 3, torch=True, nchan=2):
+                    nclasses=4, torch=True, nchan=2):
         self.unet = True
         if torch:
             if not TORCH_ENABLED:
@@ -128,7 +128,12 @@ class UnetModel():
         if device is None:
             sdevice, gpu = assign_device(torch, gpu)
         self.device = device if device is not None else sdevice
-        self.gpu = gpu if device is None else (self.device.type=='cuda')
+        if device is not None:
+            if torch:
+                device_gpu = self.device.type=='cuda'
+            else:
+                device_gpu = self.device.device_type=='gpu'
+        self.gpu = gpu if device is None else device_gpu
         if torch and not self.gpu:
             self.mkldnn = check_mkl(self.torch)
         self.pretrained_model = pretrained_model
@@ -154,7 +159,7 @@ class UnetModel():
             self.nbase = [nchan, 32, 64, 128, 256]
             self.net = resnet_torch.CPnet(self.nbase, 
                                           self.nclasses, 
-                                          3,
+                                          sz=3,
                                           residual_on=residual_on, 
                                           style_on=style_on,
                                           concatenation=concatenation,
@@ -239,12 +244,13 @@ class UnetModel():
             flows: list of lists 2D arrays, or list of 3D arrays (if do_3D=True)
                 flows[k][0] = XY flow in HSV 0-255
                 flows[k][1] = flows at each pixel
-                flows[k][2] = the cell probability centered at 0.0
+                flows[k][2] = the cell distance field
+                flows[k][3] = the cell boundary
 
             styles: list of 1D arrays of length 64, or single 1D array (if do_3D=True)
                 style vector summarizing each image, also used to estimate size of objects in image
 
-        """
+        """        
         x = [transforms.convert_image(xi, channels, channel_axis, z_axis, do_3D, 
                                     normalize, invert, nchan=self.nchan) for xi in x]
         nimg = len(x)
@@ -313,7 +319,7 @@ class UnetModel():
 
         if nolist:
             masks, flows, styles = masks[0], flows[0], styles[0]
-
+        
         return masks, flows, styles
 
     def _to_device(self, x):
@@ -405,6 +411,7 @@ class UnetModel():
                 if progress is not None:
                     progress.setValue(10 + 10*j)
             y = y / len(self.pretrained_model)
+            
         return y, style
 
     def _run_net(self, imgs, augment=False, tile=True, tile_overlap=0.1, bsize=224,
@@ -458,6 +465,7 @@ class UnetModel():
         # pad image for net so Ly and Lx are divisible by 4
         imgs, ysub, xsub = transforms.pad_image_ND(imgs)
         # slices from padding
+#         slc = [slice(0, self.nclasses) for n in range(imgs.ndim)] # changed from imgs.shape[n]+1 for first slice size 
         slc = [slice(0, imgs.shape[n]+1) for n in range(imgs.ndim)]
         slc[-3] = slice(0, self.nclasses + 32*return_conv + 1)
         slc[-2] = slice(ysub[0], ysub[-1]+1)
@@ -477,10 +485,10 @@ class UnetModel():
 
         # slice out padding
         y = y[slc]
-
         # transpose so channels axis is last again
         y = np.transpose(y, detranspose)
-         
+        
+        torch.cuda.empty_cache() #release cuda memory
         return y, style
     
     def _run_tiled(self, imgi, augment=False, bsize=224, tile_overlap=0.1, return_conv=False):
@@ -515,7 +523,6 @@ class UnetModel():
             1D array summarizing the style of the image, averaged over tiles
 
         """
-
         if imgi.ndim==4:
             batch_size = self.batch_size 
             Lz, nchan = imgi.shape[:2]
@@ -674,7 +681,7 @@ class UnetModel():
 
     def train(self, train_data, train_labels, train_files=None, 
               test_data=None, test_labels=None, test_files=None,
-              channels=None, normalize=True, pretrained_model=None, save_path=None, save_every=100,
+              channels=None, normalize=True, pretrained_model=None, save_path=None, save_every=50, save_each=False,
               learning_rate=0.2, n_epochs=500, momentum=0.9, weight_decay=0.00001, batch_size=8, rescale=False):
         """ train function uses 0-1 mask label and boundary pixels for training """
 
@@ -708,7 +715,7 @@ class UnetModel():
 
         model_path = self._train_net(train_data, train_classes, 
                                      test_data, test_classes,
-                                     pretrained_model, save_path, save_every,
+                                     pretrained_model, save_path, save_every, save_each,
                                      learning_rate, n_epochs, momentum, weight_decay, 
                                      batch_size, rescale)
 
@@ -788,10 +795,25 @@ class UnetModel():
             test_loss = nd.sum(loss).asnumpy()
         return test_loss
 
-    def _set_optimizer(self, learning_rate, momentum, weight_decay):
+    def _set_optimizer(self, learning_rate, momentum, weight_decay, SGD=False):
         if self.torch:
-            self.optimizer = optim.SGD(self.net.parameters(), lr=learning_rate,
-                            momentum=momentum, weight_decay=weight_decay)
+        # best optimizer I tested seemed to be RAdam, about 2x as fast as SGD and very stable.
+        # Ranger21 is in beta and might be better/faster, but more testing is needed.
+        # Ranger21 has a convenient current_lr field, whereas RAdam doesn't and I just set this field to the learning rate
+            if SGD:
+                self.optimizer = torch.optim.SGD(self.net.parameters(), lr=learning_rate,
+                                            momentum=momentum, weight_decay=weight_decay)
+            else:
+                self.optimizer = optim.RAdam(self.net.parameters(), lr=learning_rate, betas=(0.95, 0.999), #changed to .95
+                                            eps=1e-08, weight_decay=weight_decay)
+                core_logger.info('>>> Using RAdam optimizer')
+#             self.optimizer = optim.AdaBound(self.net.parameters(), lr=learning_rate, betas=(0.9, 0.999), 
+#                                 gamma=1e-3, eps=1e-08, final_lr=0.15, weight_decay=weight_decay)
+#             print('>>> Using AdaBound optimizer')
+#             self.optimizer = Ranger21(self.net.parameters(), lr=learning_rate, weight_decay=weight_decay, num_batches_per_epoch=self.batch_size, 
+#                                       num_epochs=self.n_epochs,num_warmup_iterations=20*self.batch_size, betas=(0.9, 0.999), eps=1e-08)
+#             print('>>> Using Ranger21 optimizer')
+            self.optimizer.current_lr = learning_rate
         else:
             self.optimizer = gluon.Trainer(self.net.collect_params(), 'sgd',{'learning_rate': learning_rate,
                                 'momentum': momentum, 'wd': weight_decay})
@@ -813,60 +835,68 @@ class UnetModel():
             if self.torch:
                 self.criterion  = nn.MSELoss(reduction='mean')
                 self.criterion2 = nn.BCEWithLogitsLoss(reduction='mean')
+                self.criterion6 = MaskedLoss()
+                self.criterion11 = DerivativeLoss()
+                self.criterion12 = WeightedLoss()
+                self.criterion14 = ArcCosDotLoss()
+                self.criterion15 = NormLoss()
+                self.criterion16 = DivergenceLoss()
             else:
                 self.criterion  = gluon.loss.L2Loss()
                 self.criterion2 = gluon.loss.SigmoidBinaryCrossEntropyLoss()
 
+    # Restored defaults. Need to make sure rescale is properly turned off and omni turned on when using CLI. 
     def _train_net(self, train_data, train_labels, 
               test_data=None, test_labels=None,
-              pretrained_model=None, save_path=None, save_every=100,
+              pretrained_model=None, save_path=None, save_every=100, save_each=False,
               learning_rate=0.2, n_epochs=500, momentum=0.9, weight_decay=0.00001, 
-              batch_size=8, rescale=True, netstr='cellpose'):
-        """ train function uses loss function self.loss_fn """
-
+              SGD=False, batch_size=8, rescale=True, netstr='cellpose'): 
+        """ train function uses loss function self.loss_fn in models.py"""
+        
         d = datetime.datetime.now()
         self.learning_rate = learning_rate
         self.n_epochs = n_epochs
         self.batch_size = batch_size
-        
-        self._set_optimizer(self.learning_rate, momentum, weight_decay)
+        self._set_optimizer(self.learning_rate, momentum, weight_decay, SGD)
         self._set_criterion()
         
-
         nimg = len(train_data)
 
         # compute average cell diameter
         if rescale:
-            diam_train = np.array([utils.diameters(train_labels[k][0])[0] for k in range(len(train_labels))])
+            diam_train = np.array([utils.diameters(train_labels[k][0],omni=self.omni)[0] for k in range(len(train_labels))])
             diam_train[diam_train<5] = 5.
             if test_data is not None:
-                diam_test = np.array([utils.diameters(test_labels[k][0])[0] for k in range(len(test_labels))])
+                diam_test = np.array([utils.diameters(test_labels[k][0],omni=self.omni)[0] for k in range(len(test_labels))])
                 diam_test[diam_test<5] = 5.
             scale_range = 0.5
+            core_logger.info('>>>> median diameter set to = %d'%self.diam_mean)
         else:
             scale_range = 1.0
 
         nchan = train_data[0].shape[0]
         core_logger.info('>>>> training network with %d channel input <<<<'%nchan)
         core_logger.info('>>>> saving every %d epochs'%save_every)
-        core_logger.info('>>>> median diameter = %d'%self.diam_mean)
         core_logger.info('>>>> LR: %0.5f, batch_size: %d, weight_decay: %0.5f'%(self.learning_rate, self.batch_size, weight_decay))
         core_logger.info('>>>> ntrain = %d'%nimg)
+        core_logger.info('>>>> rescale is %d'%rescale)
         if test_data is not None:
             core_logger.info('>>>> ntest = %d'%len(test_data))
         core_logger.info(train_data[0].shape)
-
-        # set learning rate schedule    
-        LR = np.linspace(0, self.learning_rate, 10)
-        if self.n_epochs > 250:
-            LR = np.append(LR, self.learning_rate*np.ones(self.n_epochs-100))
-            for i in range(10):
-                LR = np.append(LR, LR[-1]/2 * np.ones(10))
-        else:
-            LR = np.append(LR, self.learning_rate*np.ones(max(0,self.n_epochs-10)))
         
         tic = time.time()
 
+        # set learning rate schedule    
+        if SGD:
+            LR = np.linspace(0, self.learning_rate, 10)
+            if self.n_epochs > 250:
+                LR = np.append(LR, self.learning_rate*np.ones(self.n_epochs-100))
+                for i in range(10):
+                    LR = np.append(LR, LR[-1]/2 * np.ones(10))
+            else:
+                LR = np.append(LR, self.learning_rate*np.ones(max(0,self.n_epochs-10)))
+        
+
         lavg, nsum = 0, 0
 
         if save_path is not None:
@@ -885,16 +915,20 @@ class UnetModel():
         self.net.mkldnn = False
 
         for iepoch in range(self.n_epochs):
+                
             np.random.seed(iepoch)
             rperm = np.random.permutation(nimg)
-            self._set_learning_rate(LR[iepoch])
-
+            if SGD:
+                self._set_learning_rate(LR[iepoch])
+            
             for ibatch in range(0,nimg,batch_size):
                 inds = rperm[ibatch:ibatch+batch_size]
                 rsc = diam_train[inds] / self.diam_mean if rescale else np.ones(len(inds), np.float32)
+                # now passing in the full train array, need the labels for distance field
                 imgi, lbl, scale = transforms.random_rotate_and_resize(
-                                        [train_data[i] for i in inds], Y=[train_labels[i][1:] for i in inds],
-                                        rescale=rsc, scale_range=scale_range, unet=self.unet)
+                                        [train_data[i] for i in inds], Y=[train_labels[i] for i in inds],
+                                        rescale=rsc, scale_range=scale_range, unet=self.unet, inds=inds, omni=self.omni)
+
                 if self.unet and lbl.shape[1]>1 and rescale:
                     lbl[:,1] /= diam_batch[:,np.newaxis,np.newaxis]**2
                 train_loss = self._train_step(imgi, lbl)
@@ -911,9 +945,8 @@ class UnetModel():
                         inds = rperm[ibatch:ibatch+batch_size]
                         rsc = diam_test[inds] / self.diam_mean if rescale else np.ones(len(inds), np.float32)
                         imgi, lbl, scale = transforms.random_rotate_and_resize(
-                                            [test_data[i] for i in inds],
-                                            Y=[test_labels[i][1:] for i in inds],
-                                            scale_range=0., rescale=rsc, unet=self.unet)
+                                            [test_data[i] for i in inds], Y=[test_labels[i] for i in inds], 
+                                            scale_range=0., rescale=rsc, unet=self.unet, inds=inds, omni=self.omni) 
                         if self.unet and lbl.shape[1]>1 and rescale:
                             lbl[:,1] *= scale[0]**2
 
@@ -922,16 +955,22 @@ class UnetModel():
                         nsum += len(imgi)
 
                     core_logger.info('Epoch %d, Time %4.1fs, Loss %2.4f, Loss Test %2.4f, LR %2.4f'%
-                            (iepoch, time.time()-tic, lavg, lavgt/nsum, LR[iepoch]))
+                            (iepoch, time.time()-tic, lavg, lavgt/nsum, self.optimizer.current_lr))
                 else:
                     core_logger.info('Epoch %d, Time %4.1fs, Loss %2.4f, LR %2.4f'%
-                            (iepoch, time.time()-tic, lavg, LR[iepoch]))
+                            (iepoch, time.time()-tic, lavg, self.optimizer.current_lr))
+                
                 lavg, nsum = 0, 0
-
+                            
             if save_path is not None:
                 if iepoch==self.n_epochs-1 or iepoch%save_every==1:
                     # save model at the end
-                    file_name = '{}_{}_{}'.format(self.net_type, file_label, d.strftime("%Y_%m_%d_%H_%M_%S.%f"))
+                    if save_each: #separate files as model progresses 
+                        file_name = '{}_{}_{}_{}'.format(self.net_type, file_label, 
+                                                         d.strftime("%Y_%m_%d_%H_%M_%S.%f"),
+                                                         'epoch_'+str(iepoch)) 
+                    else:
+                        file_name = '{}_{}_{}'.format(self.net_type, file_label, d.strftime("%Y_%m_%d_%H_%M_%S.%f"))
                     file_name = os.path.join(file_path, file_name)
                     ksave += 1
                     core_logger.info(f'saving network parameters to {file_name}')
@@ -941,3 +980,101 @@ class UnetModel():
         self.net.mkldnn = self.mkldnn
 
         return file_name
+
+class DerivativeLoss(torch.nn.Module):
+    def __init__(self):
+        super().__init__()
+
+    def forward(self,y,Y,w,mask):
+        dx,dy = derivatives(y)
+        gx,gy = derivatives(Y)
+        d1 = (dx-gx)/5.
+        d2 = (dy-gy)/5.
+        L1 = torch.square(d1)
+        L2 = torch.square(d2)
+        return torch.mean((L1[mask]+L2[mask])*w[mask])
+#         return torch.mean(torch.sum((L1+L2)*w,axis=(-2,-1))/torch.sum(mask,axis=(-2,-1)))
+    
+    
+class WeightedLoss(torch.nn.Module):
+    def __init__(self):
+        super().__init__()
+
+    def forward(self,y,Y,w):
+
+        diff = (y-Y)/5.
+        return torch.mean(torch.square(diff)*w)
+
+class MaskedLoss(torch.nn.Module):
+    def __init__(self):
+        super().__init__()
+
+    def forward(self,y,Y,mask):
+        diff = (y-Y)/5.
+        return torch.mean(torch.square(diff[mask]))
+#         return torch.mean(torch.sum(torch.square(diff),axis=(-2,-1))/torch.sum(mask,axis=(-2,-1)))
+        
+def derivatives(x):
+    sobely = [[-1, -2, -1], [0, 0, 0], [1, 2, 1]]
+    sobelx = [[-1, 0, 1], [-2, 0, 2], [-1, 0, 1]]
+    depth = x.size()[1]
+    sobel_kernel_x = torch.tensor(sobelx, dtype=torch.float32).unsqueeze(0).expand(depth,1,3,3).to(torch.device('cuda'))
+    sobel_kernel_y = torch.tensor(sobely, dtype=torch.float32).unsqueeze(0).expand(depth,1,3,3).to(torch.device('cuda'))
+
+    dx = torch.nn.functional.conv2d(x, sobel_kernel_x, stride=1, padding=1, groups=x.size(1))
+    dy = torch.nn.functional.conv2d(x, sobel_kernel_y, stride=1, padding=1, groups=x.size(1))
+
+    return dy,dx 
+
+class ArcCosDotLoss(torch.nn.Module):
+    def __init__(self):
+        super().__init__()
+
+    def forward(self,x,y,w,mask):
+        eps = 1e-12
+        denom = torch.multiply(torch.linalg.norm(x,dim=1),torch.linalg.norm(y,dim=1))+eps
+        dot = (x[:,0,:,:]*y[:,0,:,:]+x[:,1,:,:]*y[:,1,:,:])
+        phasediff = torch.acos(torch.clip(dot/denom,-0.999999,0.999999))/3.141549
+        return torch.mean((torch.square(phasediff[mask]))*w[mask])
+#         return torch.mean(torch.sum(torch.square(phasediff)*w,axis=(-2,-1))/torch.sum(mask,axis=(-2,-1)))
+    
+class NormLoss(torch.nn.Module):
+    def __init__(self):
+        super().__init__()
+
+    def forward(self,y,Y,w,mask):
+        ny = torch.linalg.norm(y,dim=1,keepdim=False)/5.
+        nY = torch.linalg.norm(Y,dim=1,keepdim=False)/5.
+        diff = (ny-nY)
+        return torch.mean(torch.square(diff[mask])*w[mask])
+#         return torch.mean(torch.sum(torch.square(diff)*w,axis=(-2,-1))/torch.sum(mask,axis=(-2,-1))) #mean of means, alterna
+    
+class DivergenceLoss(torch.nn.Module):
+    def __init__(self):
+        super().__init__()
+
+    def forward(self,y,Y,mask=None):
+        divy = divergence(y)
+        divY = divergence(Y)
+        if mask is None:
+            mask = torch.abs(divY)>1
+        diff = (divY - divy)/5.
+        return torch.mean(torch.square(diff[mask]))
+#         return torch.mean(torch.sum(torch.square(diff),axis=(-2,-1))/torch.sum(mask,axis=(-2,-1)))
+
+def divergence(x):
+    sobely = [[-1, -2, -1], [0, 0, 0], [1, 2, 1]]
+    sobelx = [[-1, 0, 1], [-2, 0, 2], [-1, 0, 1]]
+    depth = x.size()[1]
+    sobel_kernel_x = torch.tensor(sobelx, dtype=torch.float32).unsqueeze(0).expand(depth,1,3,3).to(torch.device('cuda'))
+    sobel_kernel_y = torch.tensor(sobely, dtype=torch.float32).unsqueeze(0).expand(depth,1,3,3).to(torch.device('cuda'))
+
+    dx = torch.nn.functional.conv2d(x, sobel_kernel_x, stride=1, padding=1, groups=x.size(1))
+    dy = torch.nn.functional.conv2d(x, sobel_kernel_y, stride=1, padding=1, groups=x.size(1))
+    div = dy[:,0,:,:]+dx[:,1,:,:]
+    return div
+
+# averaging the mean across each 
+def mean_of_means(x):
+    return torch.mean(torch.mean(x, axis=(-2,-1)))
+
