@@ -4,6 +4,7 @@ from natsort import natsorted
 from tqdm import tqdm
 
 from PyQt5 import QtGui, QtCore, Qt, QtWidgets
+from PyQt5.QtWidgets import QMainWindow, QApplication, QWidget, QScrollBar, QSlider, QComboBox, QGridLayout, QPushButton, QFrame, QCheckBox, QLabel, QProgressBar, QLineEdit
 import pyqtgraph as pg
 from pyqtgraph import GraphicsScene
 
@@ -12,10 +13,10 @@ import cv2
 from scipy.ndimage import gaussian_filter
 
 from . import guiparts, menus, io
-from .. import models
-from ..utils import download_url_to_file, masks_to_outlines, normalize99
+from .. import models, core, dynamics
+from ..utils import download_url_to_file, masks_to_outlines
 from ..io import save_server
-from ..transforms import resize_image
+from ..transforms import resize_image, normalize99 #fixed import
 from ..plot import disk
 
 try:
@@ -32,11 +33,14 @@ try:
 except:
     SERVER_UPLOAD = False
 
-class QHLine(QtGui.QFrame):
+#Define possible models; can we make a master list in another file to use in models and main? 
+MODEL_NAMES = ['cyto', 'nuclei', 'cyto2', 'cyto2_omni', 'bact_omni']
+    
+class QHLine(QFrame):
     def __init__(self):
         super(QHLine, self).__init__()
-        self.setFrameShape(QtGui.QFrame.HLine)
-        self.setFrameShadow(QtGui.QFrame.Sunken)
+        self.setFrameShape(QFrame.HLine)
+        self.setFrameShadow(QFrame.Sunken)
 
 def avg3d(C):
     """ smooth value of c across nearby points
@@ -115,7 +119,7 @@ def make_cmap(cm=0):
 def run(image=None):
     # Always start by initializing Qt (only once per application)
     warnings.filterwarnings("ignore")
-    app = QtGui.QApplication(sys.argv)
+    app = QApplication(sys.argv)
     icon_path = pathlib.Path.home().joinpath('.cellpose', 'logo.png')
     guip_path = pathlib.Path.home().joinpath('.cellpose', 'cellpose_gui.png')
     if not icon_path.is_file():
@@ -136,7 +140,7 @@ def run(image=None):
     app.setWindowIcon(app_icon)
     os.environ['MXNET_CUDNN_AUTOTUNE_DEFAULT'] = '0'
 
-    models.download_model_weights()
+    # models.download_model_weights() # does not exist
     MainW(image=image)
     ret = app.exec_()
     sys.exit(ret)
@@ -148,7 +152,7 @@ def get_unique_points(set):
     set = list(np.unique(cps, axis=0))
     return set
 
-class MainW(QtGui.QMainWindow):
+class MainW(QMainWindow):
     def __init__(self, image=None):
         super(MainW, self).__init__()
 
@@ -187,8 +191,8 @@ class MainW(QtGui.QMainWindow):
         self.loaded = False
 
         # ---- MAIN WIDGET LAYOUT ---- #
-        self.cwidget = QtGui.QWidget(self)
-        self.l0 = QtGui.QGridLayout()
+        self.cwidget = QWidget(self)
+        self.l0 = QGridLayout()
         self.cwidget.setLayout(self.l0)
         self.setCentralWidget(self.cwidget)
         self.l0.setVerticalSpacing(6)
@@ -255,17 +259,17 @@ class MainW(QtGui.QMainWindow):
                         "selection-background-color: rgb(50,100,50);")
         self.checkstyle = "color: rgb(190,190,190);"
 
-        label = QtGui.QLabel('Views:')#[\u2191 \u2193]')
+        label = QLabel('Views:')#[\u2191 \u2193]')
         label.setStyleSheet(self.headings)
         label.setFont(self.boldfont)
         self.l0.addWidget(label, 0,0,1,1)
 
-        label = QtGui.QLabel('[up/down or W/S]')
+        label = QLabel('[up/down or W/S]')
         label.setStyleSheet(label_style)
         label.setFont(self.smallfont)
         self.l0.addWidget(label, 1,0,1,1)
 
-        label = QtGui.QLabel('[pageup/down]')
+        label = QLabel('[pageup/down]')
         label.setStyleSheet(label_style)
         label.setFont(self.smallfont)
         self.l0.addWidget(label, 1,1,1,1)
@@ -274,7 +278,7 @@ class MainW(QtGui.QMainWindow):
         self.view = 0 # 0=image, 1=flowsXY, 2=flowsZ, 3=cellprob
         self.color = 0 # 0=RGB, 1=gray, 2=R, 3=G, 4=B
         self.RGBChoose = guiparts.RGBRadioButtons(self, b,1)
-        self.RGBDropDown = QtGui.QComboBox()
+        self.RGBDropDown = QComboBox()
         self.RGBDropDown.addItems(["RGB","gray","spectral","red","green","blue"])
         self.RGBDropDown.setFont(self.medfont)
         self.RGBDropDown.currentIndexChanged.connect(self.color_choose)
@@ -292,21 +296,21 @@ class MainW(QtGui.QMainWindow):
         line.setStyleSheet('color: white;')
         self.l0.addWidget(line, b,0,1,2)
         b+=1
-        label = QtGui.QLabel('Drawing:')
+        label = QLabel('Drawing:')
         label.setStyleSheet(self.headings)
         label.setFont(self.boldfont)
         self.l0.addWidget(label, b,0,1,2)
 
         b+=1
         self.brush_size = 3
-        self.BrushChoose = QtGui.QComboBox()
+        self.BrushChoose = QComboBox()
         self.BrushChoose.addItems(["1","3","5","7","9"])
         self.BrushChoose.currentIndexChanged.connect(self.brush_choose)
         self.BrushChoose.setFixedWidth(60)
         self.BrushChoose.setStyleSheet(self.dropdowns)
         self.BrushChoose.setFont(self.medfont)
         self.l0.addWidget(self.BrushChoose, b, 1,1,1)
-        label = QtGui.QLabel('brush size: [, .]')
+        label = QLabel('brush size: [, .]')
         label.setStyleSheet(label_style)
         label.setFont(self.medfont)
         self.l0.addWidget(label, b,0,1,1)
@@ -317,7 +321,7 @@ class MainW(QtGui.QMainWindow):
 
         b+=1
         # turn on draw mode
-        self.SCheckBox = QtGui.QCheckBox('single stroke')
+        self.SCheckBox = QCheckBox('single stroke')
         self.SCheckBox.setStyleSheet(self.checkstyle)
         self.SCheckBox.setFont(self.medfont)
         self.SCheckBox.toggled.connect(self.autosave_on)
@@ -325,7 +329,7 @@ class MainW(QtGui.QMainWindow):
 
         b+=1
         # turn on crosshairs
-        self.CHCheckBox = QtGui.QCheckBox('cross-hairs')
+        self.CHCheckBox = QCheckBox('cross-hairs')
         self.CHCheckBox.setStyleSheet(self.checkstyle)
         self.CHCheckBox.setFont(self.medfont)
         self.CHCheckBox.toggled.connect(self.cross_hairs)
@@ -335,7 +339,7 @@ class MainW(QtGui.QMainWindow):
         # turn off masks
         self.layer_off = False
         self.masksOn = True
-        self.MCheckBox = QtGui.QCheckBox('MASKS ON [X]')
+        self.MCheckBox = QCheckBox('MASKS ON [X]')
         self.MCheckBox.setStyleSheet(self.checkstyle)
         self.MCheckBox.setFont(self.medfont)
         self.MCheckBox.setChecked(True)
@@ -344,17 +348,18 @@ class MainW(QtGui.QMainWindow):
 
         b+=1
         # turn off outlines
-        self.outlinesOn = True
-        self.OCheckBox = QtGui.QCheckBox('outlines on [Z]')
+        self.outlinesOn = False # turn off by default
+        self.OCheckBox = QCheckBox('outlines on [Z]')
         self.OCheckBox.setStyleSheet(self.checkstyle)
         self.OCheckBox.setFont(self.medfont)
-        self.OCheckBox.setChecked(True)
-        self.OCheckBox.toggled.connect(self.toggle_masks)
         self.l0.addWidget(self.OCheckBox, b,0,1,2)
-
+        
+        self.OCheckBox.setChecked(False)
+        self.OCheckBox.toggled.connect(self.toggle_masks) 
+        
         b+=1
         # send to server
-        self.ServerButton = QtGui.QPushButton(' send manual seg. to server')
+        self.ServerButton = QPushButton(' send manual seg. to server')
         self.ServerButton.clicked.connect(lambda: save_server(self))
         self.l0.addWidget(self.ServerButton, b,0,1,2)
         self.ServerButton.setEnabled(False)
@@ -366,19 +371,19 @@ class MainW(QtGui.QMainWindow):
         line.setStyleSheet('color: white;')
         self.l0.addWidget(line, b,0,1,2)
         b+=1
-        label = QtGui.QLabel('Segmentation:')
+        label = QLabel('Segmentation:')
         label.setStyleSheet(self.headings)
         label.setFont(self.boldfont)
         self.l0.addWidget(label, b,0,1,2)
 
         b+=1
         self.diameter = 30
-        label = QtGui.QLabel('cell diameter (pixels) (click ENTER):')
+        label = QLabel('cell diameter (pixels) (click ENTER):')
         label.setStyleSheet(label_style)
         label.setFont(self.medfont)
         label.setToolTip('you can manually enter the approximate diameter for your cells, \nor press “calibrate” to let the model estimate it. \nThe size is represented by a disk at the bottom of the view window \n(can turn this disk off by unchecking “scale disk on”)')
         self.l0.addWidget(label, b, 0,1,2)
-        self.Diameter = QtGui.QLineEdit()
+        self.Diameter = QLineEdit()
         self.Diameter.setToolTip('you can manually enter the approximate diameter for your cells, \nor press “calibrate” to let the model estimate it. \nThe size is represented by a disk at the bottom of the view window \n(can turn this disk off by unchecking “scale disk on”)')
         self.Diameter.setText(str(self.diameter))
         self.Diameter.setFont(self.medfont)
@@ -388,7 +393,7 @@ class MainW(QtGui.QMainWindow):
         self.l0.addWidget(self.Diameter, b, 0,1,2)
 
         # recompute model
-        self.SizeButton = QtGui.QPushButton('  calibrate')
+        self.SizeButton = QPushButton('  calibrate')
         self.SizeButton.clicked.connect(self.calibrate_size)
         self.l0.addWidget(self.SizeButton, b,1,1,1)
         self.SizeButton.setEnabled(False)
@@ -398,7 +403,7 @@ class MainW(QtGui.QMainWindow):
         # scale toggle
         b+=1
         self.scale_on = True
-        self.ScaleOn = QtGui.QCheckBox('scale disk on')
+        self.ScaleOn = QCheckBox('scale disk on')
         self.ScaleOn.setFont(self.medfont)
         self.ScaleOn.setStyleSheet('color: red;')
         self.ScaleOn.setChecked(True)
@@ -408,7 +413,7 @@ class MainW(QtGui.QMainWindow):
 
         # use GPU
         b+=1
-        self.useGPU = QtGui.QCheckBox('use GPU')
+        self.useGPU = QCheckBox('use GPU')
         self.useGPU.setStyleSheet(self.checkstyle)
         self.useGPU.setFont(self.medfont)
         self.useGPU.setToolTip('if you have specially installed the <i>cuda</i> version of mxnet, then you can activate this, but it won’t give huge speedups when running single 2D images in the GUI.')
@@ -416,7 +421,7 @@ class MainW(QtGui.QMainWindow):
         self.l0.addWidget(self.useGPU, b,0,1,1)
 
         # fast mode
-        self.NetAvg = QtGui.QComboBox()
+        self.NetAvg = QComboBox()
         self.NetAvg.addItems(['average 4 nets', '+ resample (slow)', 'run 1 net (fast)', ])
         self.NetAvg.setFont(self.medfont)
         self.NetAvg.setToolTip('average 4 different fit networks (default) + resample for smooth masks (slow) or run 1 network (fast)')
@@ -424,24 +429,25 @@ class MainW(QtGui.QMainWindow):
 
         b+=1
         # choose models
-        self.ModelChoose = QtGui.QComboBox()
-        self.model_dir = pathlib.Path.home().joinpath('.cellpose', 'models')
-        models = ['cyto', 'nuclei', 'cyto2']
-        self.ModelChoose.addItems(models)
+        self.ModelChoose = QComboBox()
+        self.ModelChoose.addItems(MODEL_NAMES) #added omnipose model names
         self.ModelChoose.setFixedWidth(70)
         self.ModelChoose.setStyleSheet(self.dropdowns)
         self.ModelChoose.setFont(self.medfont)
         self.l0.addWidget(self.ModelChoose, b, 1,1,1)
-        label = QtGui.QLabel('model: ')
+        label = QLabel('model: ')
         label.setStyleSheet(label_style)
         label.setFont(self.medfont)
-        label.setToolTip('there is a <em>cyto</em> model, a new <em>cyto2</em> model from user submissions, and a <em>nuclei</em> model')
-        self.ModelChoose.setToolTip('there is a <em>cyto</em> model, a new <em>cyto2</em> model from user submissions, and a <em>nuclei</em> model')
+        #update tooltip string 
+        tipstr = 'there is a <em>cyto</em> model, a new <em>cyto2</em> model from user submissions, a <em>nuclei</em> model, \
+                  and two omnipose models: <em>bact_omni</em> and <em>cyto2_omni</em>'
+        label.setToolTip(tipstr)
+        self.ModelChoose.setToolTip(tipstr)
         self.l0.addWidget(label, b, 0,1,1)
 
         b+=1
         # choose channel
-        self.ChannelChoose = [QtGui.QComboBox(), QtGui.QComboBox()]
+        self.ChannelChoose = [QComboBox(), QComboBox()]
         self.ChannelChoose[0].addItems(['gray','red','green','blue'])
         self.ChannelChoose[1].addItems(['none','red','green','blue'])
         cstr = ['chan to segment:', 'chan2 (optional): ']
@@ -449,7 +455,7 @@ class MainW(QtGui.QMainWindow):
             self.ChannelChoose[i].setFixedWidth(70)
             self.ChannelChoose[i].setStyleSheet(self.dropdowns)
             self.ChannelChoose[i].setFont(self.medfont)
-            label = QtGui.QLabel(cstr[i])
+            label = QLabel(cstr[i])
             label.setStyleSheet(label_style)
             label.setFont(self.medfont)
             if i==0:
@@ -464,36 +470,36 @@ class MainW(QtGui.QMainWindow):
 
         # use inverted image for running cellpose
         b+=1
-        self.invert = QtGui.QCheckBox('invert grayscale')
+        self.invert = QCheckBox('invert grayscale')
         self.invert.setStyleSheet(self.checkstyle)
         self.invert.setFont(self.medfont)
         self.l0.addWidget(self.invert, b,0,1,2)
 
         b+=1
         # recompute model
-        self.ModelButton = QtGui.QPushButton('  run segmentation')
+        self.ModelButton = QPushButton('  run segmentation')
         self.ModelButton.clicked.connect(self.compute_model)
         self.l0.addWidget(self.ModelButton, b,0,1,2)
         self.ModelButton.setEnabled(False)
         self.ModelButton.setStyleSheet(self.styleInactive)
         self.ModelButton.setFont(self.boldfont)
         b+=1
-        self.progress = QtGui.QProgressBar(self)
+        self.progress = QProgressBar(self)
         self.progress.setStyleSheet('color: gray;')
         self.l0.addWidget(self.progress, b,0,1,2)
 
         # post-hoc paramater tuning
 
         b+=1
-        label = QtGui.QLabel('model match threshold:')
-        label.setToolTip('threshold on gradient match to accept a mask (set lower to get more cells)')
+        label = QLabel('model match threshold:')
+        label.setToolTip('threshold on flow match to accept a mask (set lower to get more cells)')
         label.setStyleSheet(label_style)
         label.setFont(self.medfont)
         self.l0.addWidget(label, b, 0,1,2)
 
         b+=1
         self.threshold = 0.4
-        self.threshslider = QtGui.QSlider()
+        self.threshslider = QSlider()
         self.threshslider.setOrientation(QtCore.Qt.Horizontal)
         self.threshslider.setMinimum(1.0)
         self.threshslider.setMaximum(30.0)
@@ -504,14 +510,15 @@ class MainW(QtGui.QMainWindow):
         self.threshslider.setEnabled(False)
         
         b+=1
-        label = QtGui.QLabel('cell prob threshold:')
+        label = QLabel('mask threshold:')
+        label.setToolTip('threshold on scalar output field to seed cell masks \
+                        (set lower to include more pixels)')
         label.setStyleSheet(label_style)
         label.setFont(self.medfont)
         self.l0.addWidget(label, b, 0,1,2)
-        label.setToolTip('cell probability threshold (set lower to get more cells)')
         
         b+=1
-        self.probslider = QtGui.QSlider()
+        self.probslider = QSlider()
         self.probslider.setOrientation(QtCore.Qt.Horizontal)
         self.probslider.setMinimum(-6.0)
         self.probslider.setMaximum(6.0)
@@ -527,14 +534,14 @@ class MainW(QtGui.QMainWindow):
         line.setStyleSheet('color: white;')
         self.l0.addWidget(line, b,0,1,2)
 
-        self.autobtn = QtGui.QCheckBox('auto-adjust')
+        self.autobtn = QCheckBox('auto-adjust')
         self.autobtn.setStyleSheet(self.checkstyle)
         self.autobtn.setFont(self.medfont)
         self.autobtn.setChecked(True)
         self.l0.addWidget(self.autobtn, b+2,0,1,1)
 
         b+=1
-        label = QtGui.QLabel('Image saturation:')
+        label = QLabel('Image saturation:')
         label.setStyleSheet(self.headings)
         label.setFont(self.boldfont)
         self.l0.addWidget(label, b,0,1,2)
@@ -545,18 +552,18 @@ class MainW(QtGui.QMainWindow):
         self.slider.setMaximum(255)
         self.slider.setLow(0)
         self.slider.setHigh(255)
-        self.slider.setTickPosition(QtGui.QSlider.TicksRight)
+        self.slider.setTickPosition(QSlider.TicksRight)
         self.l0.addWidget(self.slider, b,1,1,1)
         self.l0.setRowStretch(b, 1)
         
         b+=2
         # add z position underneath
         self.currentZ = 0
-        label = QtGui.QLabel('Z:')
+        label = QLabel('Z:')
         label.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
         label.setStyleSheet(label_style)
         self.l0.addWidget(label, b, 0,1,1)
-        self.zpos = QtGui.QLineEdit()
+        self.zpos = QLineEdit()
         self.zpos.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
         self.zpos.setText(str(self.currentZ))
         self.zpos.returnPressed.connect(self.compute_scale)
@@ -564,7 +571,7 @@ class MainW(QtGui.QMainWindow):
         self.l0.addWidget(self.zpos, b, 1,1,1)
 
         # add scrollbar underneath
-        self.scroll = QtGui.QScrollBar(QtCore.Qt.Horizontal)
+        self.scroll = QScrollBar(QtCore.Qt.Horizontal)
         self.scroll.setMaximum(10)
         self.scroll.valueChanged.connect(self.move_in_Z)
         self.l0.addWidget(self.scroll, b,3,1,20)
@@ -660,11 +667,11 @@ class MainW(QtGui.QMainWindow):
         self.torch = torch
         self.useGPU.setChecked(False)
         self.useGPU.setEnabled(False)    
-        if self.torch and models.use_gpu(istorch=True):
+        if self.torch and core.use_gpu(istorch=True):
             self.useGPU.setEnabled(True)
             self.useGPU.setChecked(True)
         elif models.MXNET_ENABLED:
-            if models.use_gpu(istorch=False):
+            if core.use_gpu(istorch=False):
                 print('>>> will run model on GPU in mxnet <<<')
                 self.torch = False
                 self.useGPU.setEnabled(True)
@@ -1247,8 +1254,9 @@ class MainW(QtGui.QMainWindow):
         # compute percentiles from stack
         self.saturation = []
         for n in range(len(self.stack)):
-            self.saturation.append([np.percentile(self.stack[n].astype(np.float32),1),
-                                    np.percentile(self.stack[n].astype(np.float32),99)])
+            # changed to use omnipose convention 
+            self.saturation.append([np.percentile(self.stack[n].astype(np.float32),0.01),
+                                    np.percentile(self.stack[n].astype(np.float32),99.99)])
 
     def chanchoose(self, image):
         if image.ndim > 2:
@@ -1288,17 +1296,18 @@ class MainW(QtGui.QMainWindow):
             print('computing masks with cell prob=%0.3f, flow error threshold=%0.3f'%
                     (self.cellprob, thresh))
 
-        maski = self.model.cp._compute_masks(self.flows[4][:-1],
-                                             self.flows[4][-1],
-                                             p=self.flows[3].copy(),
-                                             cellprob_threshold=self.cellprob,
-                                             flow_threshold=thresh,
-                                             resize=self.cellpix.shape[-2:])[0]
+        maski = dynamics.compute_masks(self.flows[4][:-1], 
+                                       self.flows[4][-1],
+                                       p=self.flows[3].copy(),
+                                       mask_threshold=self.cellprob,
+                                       flow_threshold=thresh,
+                                       resize=self.cellpix.shape[-2:],
+                                       omni=self.omni)[0]
         
         self.masksOn = True
-        self.outlinesOn = True
         self.MCheckBox.setChecked(True)
-        self.OCheckBox.setChecked(True)
+        # self.outlinesOn = True #should not turn outlines back on by default; masks make sense though 
+        # self.OCheckBox.setChecked(True)
         if maski.ndim<3:
             maski = maski[np.newaxis,...]
         print('%d cells found'%(len(np.unique(maski)[1:])))
@@ -1324,12 +1333,21 @@ class MainW(QtGui.QMainWindow):
             channels = self.get_channels()
             self.diameter = float(self.Diameter.text())
             try:
+                self.omni = 'omni' in self.current_model
+                bacterial = 'bact' in self.current_model
+                if self.omni:
+                    self.NetAvg.setCurrentIndex(2) #one run net
+                if bacterial:
+                    self.diameter = 0.
+                    
                 net_avg = self.NetAvg.currentIndex()<2
                 resample = self.NetAvg.currentIndex()==1
+                # print(data.shape,channels,self.diameter,resample,do_3D,net_avg)
+                print('net_avg',net_avg)
                 masks, flows, _, _ = self.model.eval(data, channels=channels,
                                                     diameter=self.diameter, invert=self.invert.isChecked(),
                                                     net_avg=net_avg, augment=False, resample=resample,
-                                                    do_3D=do_3D, progress=self.progress)
+                                                    do_3D=do_3D, progress=self.progress, omni=self.omni)
             except Exception as e:
                 print('NET ERROR: %s'%e)
                 self.progress.setValue(0)
@@ -1340,8 +1358,8 @@ class MainW(QtGui.QMainWindow):
             #if not do_3D:
             #    masks = masks[0][np.newaxis,:,:]
             #    flows = flows[0]
-            self.flows[0] = flows[0]
-            self.flows[1] = (np.clip(normalize99(flows[2].copy()),0,1) * 255).astype(np.uint8)
+            self.flows[0] = (normalize99(flows[0].copy(),omni=True) * 255).astype(np.uint8) #RGB flow
+            self.flows[1] = (normalize99(flows[2].copy(),omni=True) * 255).astype(np.uint8) #dist/prob
             if not do_3D:
                 masks = masks[np.newaxis,...]
                 self.flows[0] = resize_image(self.flows[0], masks.shape[-2], masks.shape[-1],
@@ -1353,17 +1371,17 @@ class MainW(QtGui.QMainWindow):
             else:
                 self.flows[2] = (flows[1][0]/10 * 127 + 127).astype(np.uint8)
                 
-            if len(flows)>2:
-                self.flows.append(flows[3].squeeze())
-                self.flows.append(np.concatenate((flows[1], flows[2][np.newaxis,...]), axis=0))
+            if len(flows)>2: 
+                self.flows.append(flows[4].squeeze()) #p 
+                self.flows.append(np.concatenate((flows[1], flows[2][np.newaxis,...]), axis=0)) #dP, dist/prob
                 
             print('%d cells found with cellpose net in %0.3f sec'%(len(np.unique(masks)[1:]), time.time()-tic))
             self.progress.setValue(80)
             z=0
             self.masksOn = True
-            self.outlinesOn = True
             self.MCheckBox.setChecked(True)
-            self.OCheckBox.setChecked(True)
+            # self.outlinesOn = True #again, this option should persist and not get toggled by another GUI action 
+            # self.OCheckBox.setChecked(True)
 
             io._masks_to_gui(self, masks, outlines=None)
             self.progress.setValue(100)
